// this file is part of
// markonvert - Fortran unformatted record marker converter
//
// Copyright (C) 2005  Francesco Poli
//
// This work is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 2,
// as published by the Free Software Foundation.
// 
// This work is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this work; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

#include "config.h"
#include "engine.h"

void
mk::Engine::process_start_mark32()
{
    // read record start marker from stdin
    std::cin.read(reinterpret_cast<char*>(&record_len), sizeof(int32));
    Assert<Ex_neg_length>(record_len >= 0);

    // write record start marker to stdout
    int32 zero = 0;
    std::cout.write(reinterpret_cast<char*>(&record_len), sizeof(int32));
    std::cout.write(reinterpret_cast<char*>(&zero),       sizeof(int32));
}
   /* works for LITTLE ENDIAN data only on LITTLE ENDIAN machines only!
      TODO: the general case should be addressed... */

void
mk::Engine::process_start_mark64()
{
    // read record start marker from stdin
    int32 zero;
    std::cin.read(reinterpret_cast<char*>(&record_len), sizeof(int32));
    std::cin.read(reinterpret_cast<char*>(&zero),       sizeof(int32));
    Assert<Ex_long_record>(zero == 0);
    Assert<Ex_neg_length>(record_len >= 0);

    // write record start marker to stdout
    std::cout.write(reinterpret_cast<char*>(&record_len), sizeof(int32));
}
   /* works for LITTLE ENDIAN data only on LITTLE ENDIAN machines only!
      TODO: the general case should be addressed... */

void
mk::Engine::copy_record()
{
    char byte;
    for (int32 i = 0; i < record_len; ++i)
      {
        std::cin.get(byte);
        std::cout.put(byte);
      }
}

void
mk::Engine::process_end_mark32()
{
    // read record end marker from stdin
    int32 end;
    std::cin.read(reinterpret_cast<char*>(&end), sizeof(int32));
    Assert<Ex_marks_diff>(end == record_len);

    // write record start marker to stdout
    int32 zero = 0;
    std::cout.write(reinterpret_cast<char*>(&end),  sizeof(int32));
    std::cout.write(reinterpret_cast<char*>(&zero), sizeof(int32));
}
   /* works for LITTLE ENDIAN data only on LITTLE ENDIAN machines only!
      TODO: the general case should be addressed... */

void
mk::Engine::process_end_mark64()
{
    // read record end marker from stdin
    int32 end, zero;
    std::cin.read(reinterpret_cast<char*>(&end),  sizeof(int32));
    std::cin.read(reinterpret_cast<char*>(&zero), sizeof(int32));
    Assert<Ex_marks_diff>(end == record_len);
    Assert<Ex_marks_diff>(zero == 0);

    // write record start marker to stdout
    std::cout.write(reinterpret_cast<char*>(&end),  sizeof(int32));
}
   /* works for LITTLE ENDIAN data only on LITTLE ENDIAN machines only!
      TODO: the general case should be addressed... */
