// this is file is part of
// markonvert - Fortran unformatted record marker converter
//
// Copyright (C) 2005  Francesco Poli
//
// This work is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 2,
// as published by the Free Software Foundation.
// 
// This work is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this work; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

#ifndef MK_ENGINE_H
#define MK_ENGINE_H

#include <iostream>
#include "myassert.h"

typedef int int32;

namespace mk
{
    // Exceptions
    class Ex_int_not_4   { }; // int32 is not 4 byte long
    class Ex_chr_not_1   { }; // char  is not 1 byte long
    class Ex_neg_length  { }; // found negative record length
    class Ex_long_record { }; // found record longer than (pow(2,31) - 1) byte
    class Ex_marks_diff  { }; // end marker differs from start marker

    class Engine
      {
        public:
            Engine()
                : record_len(0)
              {
                Assert<Ex_int_not_4>(sizeof(int32) == 4);
                Assert<Ex_chr_not_1>(sizeof(char ) == 1);
                std::cin.exceptions(std::ios::goodbit);
              }

            void from32to64()  // convert markers from 32 bit to 64 bit
              {
                while(std::cin.peek() != EOF)
                  {
                    std::cin.exceptions(std::ios::eofbit |
                                        std::ios::badbit | std::ios::failbit);
                    process_start_mark32();
                    copy_record();
                    process_end_mark32();
                    std::cin.exceptions(std::ios::goodbit);
                  }
              }

            void from64to32()  // convert markers from 64 bit to 32 bit
              {
                while(std::cin.peek() != EOF)
                  {
                    std::cin.exceptions(std::ios::eofbit |
                                        std::ios::badbit | std::ios::failbit);
                    process_start_mark64();
                    copy_record();
                    process_end_mark64();
                    std::cin.exceptions(std::ios::goodbit);
                  }
              }

        private:
            void process_start_mark32();
            void process_start_mark64();
            void copy_record();
            void process_end_mark32();
            void process_end_mark64();

            int32 record_len;
      };
}

#endif // not MK_ENGINE_H

